@@ -3,7 +3,7 @@
  *  Tests for symbolic differentiation, including various functions. */
 
 /*
- *  GiNaC Copyright (C) 1999-2007 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2010 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,7 +20,11 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include "exams.h"
+#include "ginac.h"
+using namespace GiNaC;
+
+#include <iostream>
+using namespace std;
 
 static unsigned check_diff(const ex &e, const symbol &x,
 						   const ex &d, unsigned nth=1)
@@ -260,7 +264,6 @@ unsigned exam_differentiation()
 	unsigned result = 0;
 	
 	cout << "examining symbolic differentiation" << flush;
-	clog << "----------symbolic differentiation:" << endl;
 	
 	result += exam_differentiation1();  cout << '.' << flush;
 	result += exam_differentiation2();  cout << '.' << flush;
@@ -270,11 +273,10 @@ unsigned exam_differentiation()
 	result += exam_differentiation6();  cout << '.' << flush;
 	result += exam_differentiation7();  cout << '.' << flush;
 	
-	if (!result) {
-		cout << " passed " << endl;
-		clog << "(no output)" << endl;
-	} else {
-		cout << " failed " << endl;
-	}
 	return result;
 }
+
+int main(int argc, char** argv)
+{
+	return exam_differentiation();
+}
