@@ -6,7 +6,7 @@
  */
 
 /*
- *  GiNaC Copyright (C) 1999-2007 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2010 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -23,7 +23,12 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include "times.h"
+#include "ginac.h"
+#include "timer.h"
+using namespace GiNaC;
+
+#include <iostream>
+using namespace std;
 
 static unsigned test()
 {
@@ -50,7 +55,6 @@ unsigned time_fateman_expand()
 	double time = .0;
 
 	cout << "timing Fateman's polynomial expand benchmark" << flush;
-	clog << "-------Fateman's polynomial expand benchmark:" << endl;
 
 	concord.start();
 	// correct for very small times:
@@ -60,13 +64,16 @@ unsigned time_fateman_expand()
 	} while ((time=concord.read())<0.1 && !result);
 	cout << '.' << flush;
 
-	if (!result) {
-		cout << " passed ";
-		clog << "(no output)" << endl;
-	} else {
-		cout << " failed ";
-	}
 	cout << time/count << 's' << endl;
 
 	return result;
 }
+
+extern void randomify_symbol_serials();
+
+int main(int argc, char** argv)
+{
+	randomify_symbol_serials();
+	cout << setprecision(2) << showpoint;
+	return time_fateman_expand();
+}
