@@ -3,7 +3,7 @@
  *  Some timings on series expansion of the Gamma function around a pole. */
 
 /*
- *  GiNaC Copyright (C) 1999-2007 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2010 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,20 +20,26 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include "times.h"
+#include "ginac.h"
+#include "timer.h"
+using namespace GiNaC;
+
+#include <iostream>
+#include <vector>
+using namespace std;
 
 unsigned tgammaseries(unsigned order)
 {
 	unsigned result = 0;
 	symbol x;
 
-	ex myseries = series(tgamma(x),x==0,order);
+	ex myseries = series(GiNaC::tgamma(x),x==0,order);
 	// compute the last coefficient numerically:
 	ex last_coeff = myseries.coeff(x,order-1).evalf();
 	// compute a bound for that coefficient using a variation of the leading
 	// term in Stirling's formula:
 	ex bound = exp(-.57721566490153286*(order-1))/(order-1);
-	if (abs((last_coeff-pow(-1,order))/bound) > 1) {
+	if (abs((last_coeff-pow(-1,ex(order)))/bound) > 1) {
 		clog << "The " << order-1
 		     << "th order coefficient in the power series expansion of tgamma(0) was erroneously found to be "
 		     << last_coeff << ", violating a simple estimate." << endl;
@@ -48,7 +54,6 @@ unsigned time_gammaseries()
 	unsigned result = 0;
 
 	cout << "timing Laurent series expansion of Gamma function" << flush;
-	clog << "-------Laurent series expansion of Gamma function:" << endl;
 
 	vector<unsigned> sizes;
 	vector<double> times;
@@ -66,12 +71,6 @@ unsigned time_gammaseries()
 		cout << '.' << flush;
 	}
 
-	if (!result) {
-		cout << " passed ";
-		clog << "(no output)" << endl;
-	} else {
-		cout << " failed ";
-	}
 	// print the report:
 	cout << endl << "	order: ";
 	for (vector<unsigned>::iterator i=sizes.begin(); i!=sizes.end(); ++i)
@@ -83,3 +82,12 @@ unsigned time_gammaseries()
 	
 	return result;
 }
+
+extern void randomify_symbol_serials();
+
+int main(int argc, char** argv)
+{
+	randomify_symbol_serials();
+	cout << setprecision(2) << showpoint;
+	return time_gammaseries();
+}
