@@ -4,7 +4,7 @@
  *  Lewis and Michael Wester. */
 
 /*
- *  GiNaC Copyright (C) 1999-2007 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2010 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,7 +21,13 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include "times.h"
+#include "ginac.h"
+#include "timer.h"
+using namespace GiNaC;
+
+#include <iostream>
+#include <vector>
+using namespace std;
 
 static const bool do_test = true;  // set to true in order to run this beast
 
@@ -74,7 +80,6 @@ unsigned time_lw_Qprime()
 	double time = .0;
 	
 	cout << "timing Lewis-Wester test Q' (charpoly(P'))" << flush;
-	clog << "-------Lewis-Wester test Q' (charpoly(P')):" << endl;
 	
 	if (do_test) {
 		rolex.start();
@@ -84,18 +89,19 @@ unsigned time_lw_Qprime()
 			++count;
 		} while ((time=rolex.read())<0.1 && !result);
 		cout << '.' << flush;
-		
-		if (!result) {
-			cout << " passed ";
-			clog << "(no output)" << endl;
-		} else {
-			cout << " failed ";
-		}
 		cout << time/count << 's' << endl;
 	} else {
 		cout << " disabled" << endl;
-		clog << "(no output)" << endl;
 	}
 	
 	return result;
 }
+
+extern void randomify_symbol_serials();
+
+int main(int argc, char** argv)
+{
+	randomify_symbol_serials();
+	cout << setprecision(2) << showpoint;
+	return time_lw_Qprime();
+}
