@@ -3,7 +3,7 @@
  *  Assertion macro definition. */
 
 /*
- *  GiNaC Copyright (C) 1999-2007 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2010 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,8 +20,8 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#ifndef __GINAC_ASSERTION_H__
-#define __GINAC_ASSERTION_H__
+#ifndef GINAC_ASSERTION_H
+#define GINAC_ASSERTION_H
 
 #if !defined(GINAC_ASSERT)
 #if defined(DO_GINAC_ASSERT)
@@ -34,4 +34,4 @@
 #endif
 #endif
 
-#endif // ndef __GINAC_ASSERTION_H__
+#endif // ndef GINAC_ASSERTION_H
