@@ -3,7 +3,7 @@
  *  Interface to GiNaC's clifford algebra (Dirac gamma) objects. */
 
 /*
- *  GiNaC Copyright (C) 1999-2008 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2010 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,8 +20,8 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#ifndef __GINAC_CLIFFORD_H__
-#define __GINAC_CLIFFORD_H__
+#ifndef GINAC_CLIFFORD_H
+#define GINAC_CLIFFORD_H
 
 #include "indexed.h"
 #include "tensor.h"
@@ -32,7 +32,6 @@
 
 namespace GiNaC {
 
-
 /** This class holds an object representing an element of the Clifford
  *  algebra (the Dirac gamma matrices). These objects only carry Lorentz
  *  indices. Spinor indices are hidden. A representation label (an unsigned
@@ -41,9 +40,6 @@ namespace GiNaC {
 class clifford : public indexed
 {
 	GINAC_DECLARE_REGISTERED_CLASS(clifford, indexed)
-public:
-	static const tinfo_static_t return_type_tinfo_static[256];
-
 	// other constructors
 public:
 	clifford(const ex & b, unsigned char rl = 0);
@@ -56,14 +52,15 @@ public:
 	// functions overriding virtual functions from base classes
 public:
 	unsigned precedence() const { return 65; }
+	void archive(archive_node& n) const;
+	void read_archive(const archive_node& n, lst& sym_lst);
 protected:
 	ex eval_ncmul(const exvector & v) const;
 	bool match_same_type(const basic & other) const;
 	ex thiscontainer(const exvector & v) const;
 	ex thiscontainer(std::auto_ptr<exvector> vp) const;
 	unsigned return_type() const { return return_types::noncommutative; }
-	tinfo_t return_type_tinfo() const { return clifford::return_type_tinfo_static+representation_label; }
-
+	return_type_t return_type_tinfo() const;
 	// non-virtual functions in this class
 public:
 	unsigned char get_representation_label() const { return representation_label; }
@@ -87,6 +84,7 @@ protected:
 	ex metric; /**< Metric of the space, all constructors make it an indexed object */
 	int commutator_sign; /**< It is the sign in the definition e~i e~j +/- e~j e~i = B(i, j) + B(j, i)*/
 };
+GINAC_DECLARE_UNARCHIVER(clifford); 
 
 /** This class represents the Clifford algebra unity element. */
 class diracone : public tensor
@@ -98,6 +96,7 @@ protected:
 	void do_print(const print_context & c, unsigned level) const;
 	void do_print_latex(const print_latex & c, unsigned level) const;
 };
+GINAC_DECLARE_UNARCHIVER(diracone);
 
 
 /** This class represents the Clifford algebra generators (units). */
@@ -105,10 +104,6 @@ class cliffordunit : public tensor
 {
 	GINAC_DECLARE_REGISTERED_CLASS(cliffordunit, tensor)
 
-	// other constructors
-protected:
-	cliffordunit(tinfo_t ti) : inherited(ti) {}
-                                                                                                    
 	// functions overriding virtual functions from base classes
 public:
 	bool contract_with(exvector::iterator self, exvector::iterator other, exvector & v) const;
@@ -134,6 +129,7 @@ protected:
 	void do_print(const print_context & c, unsigned level) const;
 	void do_print_latex(const print_latex & c, unsigned level) const;
 };
+GINAC_DECLARE_UNARCHIVER(diracgamma);
 
 
 /** This class represents the Dirac gamma5 object which anticommutates with
@@ -150,6 +146,7 @@ protected:
 	void do_print(const print_context & c, unsigned level) const;
 	void do_print_latex(const print_latex & c, unsigned level) const;
 };
+GINAC_DECLARE_UNARCHIVER(diracgamma5);
 
 
 /** This class represents the Dirac gammaL object which behaves like
@@ -166,6 +163,7 @@ protected:
 	void do_print(const print_context & c, unsigned level) const;
 	void do_print_latex(const print_latex & c, unsigned level) const;
 };
+GINAC_DECLARE_UNARCHIVER(diracgammaL);
 
 
 /** This class represents the Dirac gammaL object which behaves like
@@ -182,15 +180,19 @@ protected:
 	void do_print(const print_context & c, unsigned level) const;
 	void do_print_latex(const print_latex & c, unsigned level) const;
 };
+GINAC_DECLARE_UNARCHIVER(diracgammaR);
 
 
 // global functions
 
-/** Check whether a given tinfo key (as returned by return_type_tinfo()
+/** Check whether a given return_type_t object (as returned by return_type_tinfo()
   * is that of a clifford object (with an arbitrary representation label).
   *
   * @param ti tinfo key */
-bool is_clifford_tinfo(tinfo_t ti);
+inline bool is_clifford_tinfo(const return_type_t& ti)
+{
+	return *(ti.tinfo) == typeid(clifford);
+}
 
 /** Create a Clifford unity object.
  *
@@ -358,4 +360,4 @@ ex clifford_moebius_map(const ex & M, const ex & v, const ex & G, unsigned char
 
 } // namespace GiNaC
 
-#endif // ndef __GINAC_CLIFFORD_H__
+#endif // ndef GINAC_CLIFFORD_H
