@@ -3,7 +3,7 @@
  *  Interface to GiNaC's color (SU(3) Lie algebra) objects. */
 
 /*
- *  GiNaC Copyright (C) 1999-2007 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2010 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,8 +20,8 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#ifndef __GINAC_COLOR_H__
-#define __GINAC_COLOR_H__
+#ifndef GINAC_COLOR_H
+#define GINAC_COLOR_H
 
 #include "indexed.h"
 #include "tensor.h"
@@ -30,7 +30,6 @@
 
 namespace GiNaC {
 
-
 /** This class holds a generator T_a or the unity element of the Lie algebra
  *  of SU(3), as used for calculations in quantum chromodynamics. A
  *  representation label (an unsigned 8-bit integer) is used to distinguish
@@ -41,7 +40,6 @@ namespace GiNaC {
 class color : public indexed
 {
 	GINAC_DECLARE_REGISTERED_CLASS(color, indexed)
-
 	// other constructors
 public:
 	color(const ex & b, unsigned char rl = 0);
@@ -50,6 +48,8 @@ public:
 	// internal constructors
 	color(unsigned char rl, const exvector & v, bool discardable = false);
 	color(unsigned char rl, std::auto_ptr<exvector> vp);
+	void archive(archive_node& n) const;
+	void read_archive(const archive_node& n, lst& sym_lst);
 
 	// functions overriding virtual functions from base classes
 protected:
@@ -58,7 +58,7 @@ protected:
 	ex thiscontainer(const exvector & v) const;
 	ex thiscontainer(std::auto_ptr<exvector> vp) const;
 	unsigned return_type() const { return return_types::noncommutative; }
-	unsigned return_type_tinfo() const { return TINFO_color + representation_label; }
+	return_type_t return_type_tinfo() const;
 
 	// non-virtual functions in this class
 public:
@@ -68,6 +68,7 @@ public:
 private:
 	unsigned char representation_label; /**< Representation label to distinguish independent color matrices coming from separated fermion lines */
 };
+GINAC_DECLARE_UNARCHIVER(color); 
 
 
 /** This class represents the su(3) unity element. */
@@ -80,6 +81,7 @@ protected:
 	void do_print(const print_context & c, unsigned level) const;
 	void do_print_latex(const print_latex & c, unsigned level) const;
 };
+GINAC_DECLARE_UNARCHIVER(su3one);
 
 /** This class represents an su(3) generator. */
 class su3t : public tensor
@@ -95,6 +97,7 @@ protected:
 	void do_print(const print_context & c, unsigned level) const;
 	void do_print_latex(const print_latex & c, unsigned level) const;
 };
+GINAC_DECLARE_UNARCHIVER(su3t);
 
 /** This class represents the tensor of antisymmetric su(3) structure
  *  constants. */
@@ -109,9 +112,11 @@ public:
 
 	// non-virtual functions in this class
 protected:
+	unsigned return_type() const { return return_types::commutative; }
 	void do_print(const print_context & c, unsigned level) const;
 	void do_print_latex(const print_latex & c, unsigned level) const;
 };
+GINAC_DECLARE_UNARCHIVER(su3f);
 
 /** This class represents the tensor of symmetric su(3) structure constants. */
 class su3d : public tensor
@@ -125,19 +130,15 @@ public:
 
 	// non-virtual functions in this class
 protected:
+	unsigned return_type() const { return return_types::commutative; }
 	void do_print(const print_context & c, unsigned level) const;
 	void do_print_latex(const print_latex & c, unsigned level) const;
 };
+GINAC_DECLARE_UNARCHIVER(su3d);
 
 
 // global functions
 
-/** Specialization of is_exactly_a<color>(obj) for color objects. */
-template<> inline bool is_exactly_a<color>(const basic & obj)
-{
-	return obj.tinfo()==TINFO_color;
-}
-
 /** Create the su(3) unity element. This is an indexed object, although it
  *  has no indices.
  *
@@ -190,7 +191,6 @@ ex color_trace(const ex & e, const lst & rll);
  *  @param rl Representation label */
 ex color_trace(const ex & e, unsigned char rl = 0);
 
-
 } // namespace GiNaC
 
-#endif // ndef __GINAC_COLOR_H__
+#endif // ndef GINAC_COLOR_H
