@@ -6,7 +6,7 @@
  */
 
 /*
- *  GiNaC Copyright (C) 1999-2008 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2010 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -25,27 +25,26 @@
 
 #include "excompiler.h"
 
-#include <stdexcept>
-#include <ios>
-#include <fstream>
-#include <sstream>
-#include <string>
-#include <vector>
-
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
-#ifdef HAVE_LIBDL
-#include <dlfcn.h>
-#endif // def HAVE_LIBDL
-
 #include "ex.h"
 #include "lst.h"
 #include "operators.h"
 #include "relational.h"
 #include "symbol.h"
 
+#ifdef HAVE_LIBDL
+#include <dlfcn.h>
+#endif // def HAVE_LIBDL
+#include <fstream>
+#include <ios>
+#include <sstream>
+#include <stdexcept>
+#include <string>
+#include <vector>
+
 namespace GiNaC {
 
 #ifdef HAVE_LIBDL
@@ -114,12 +113,12 @@ public:
 			char* new_filename = new char[strlen(filename_pattern)+1];
 			strcpy(new_filename, filename_pattern);
 			if (!mktemp(new_filename)) {
-				delete new_filename;
+				delete[] new_filename;
 				throw std::runtime_error("mktemp failed");
 			}
 			filename = std::string(new_filename);
 			ofs.open(new_filename, std::ios::out);
-			delete new_filename;
+			delete[] new_filename;
 		} else {
 			// use parameter as filename
 			ofs.open(filename.c_str(), std::ios::out);
@@ -173,7 +172,7 @@ public:
 		for (std::vector<filedesc>::iterator it = filelist.begin(); it != filelist.end();) {
 			if (it->name == filename) {
 				clean_up(it);
-				filelist.erase(it);
+				it = filelist.erase(it);
 			} else {
 				++it;
 			}
@@ -246,14 +245,14 @@ void compile_ex(const ex& expr, const symbol& sym1, const symbol& sym2, FUNCP_2P
 void compile_ex(const lst& exprs, const lst& syms, FUNCP_CUBA& fp, const std::string filename)
 {
 	lst replacements;
-	for (int count=0; count<syms.nops(); ++count) {
+	for (std::size_t count=0; count<syms.nops(); ++count) {
 		std::ostringstream s;
 		s << "a[" << count << "]";
 		replacements.append(syms.op(count) == symbol(s.str()));
 	}
 
 	std::vector<ex> expr_with_cname;
-	for (int count=0; count<exprs.nops(); ++count) {
+	for (std::size_t count=0; count<exprs.nops(); ++count) {
 		expr_with_cname.push_back(exprs.op(count).subs(replacements));
 	}
 
@@ -263,7 +262,7 @@ void compile_ex(const lst& exprs, const lst& syms, FUNCP_CUBA& fp, const std::st
 
 	ofs << "void compiled_ex(const int* an, const double a[], const int* fn, double f[])" << std::endl;
 	ofs << "{" << std::endl;
-	for (int count=0; count<exprs.nops(); ++count) {
+	for (std::size_t count=0; count<exprs.nops(); ++count) {
 		ofs << "f[" << count << "] = ";
 		expr_with_cname[count].print(GiNaC::print_csrc_double(ofs));
 		ofs << ";" << std::endl;
