@@ -2,7 +2,7 @@
 
 #  function.pl options: \$maxargs=${maxargs}
 # 
-#  GiNaC Copyright (C) 1999-2009 Johannes Gutenberg University Mainz, Germany
+#  GiNaC Copyright (C) 1999-2010 Johannes Gutenberg University Mainz, Germany
 # 
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -292,7 +292,7 @@ $interface=<<END_OF_INTERFACE;
  *  Please do not modify it directly, edit the perl script instead!
  *  function.pl options: \$maxargs=${maxargs}
  *
- *  GiNaC Copyright (C) 1999-2009 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2010 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -574,7 +574,7 @@ $implementation=<<END_OF_IMPLEMENTATION;
  *  Please do not modify it directly, edit the perl script instead!
  *  function.pl options: \$maxargs=${maxargs}
  *
- *  GiNaC Copyright (C) 1999-2009 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2010 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -1124,7 +1124,7 @@ ex function::conjugate() const
 	const function_options & opt = registered_functions()[serial];
 
 	if (opt.conjugate_f==0) {
-		return exprseq::conjugate();
+		return conjugate_function(*this).hold();
 	}
 
 	if (opt.conjugate_use_exvector_args) {
@@ -1314,8 +1314,8 @@ ex function::power(const ex & power_param) const // power of function
 	
 	// No derivative defined? Then return abstract derivative object
 	if (opt.power_f == NULL)
-		return (new power::power(*this, power_param))->setflag(status_flags::dynallocated |
-	                                               status_flags::evaluated);
+		return (new GiNaC::power(*this, power_param))->setflag(status_flags::dynallocated |
+		                                                       status_flags::evaluated);
 
 	current_serial = serial;
 	if (opt.power_use_exvector_args)
