@@ -3,7 +3,7 @@
  *  Implementation of GiNaC's indices. */
 
 /*
- *  GiNaC Copyright (C) 1999-2008 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2010 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,10 +20,6 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <iostream>
-#include <sstream>
-#include <stdexcept>
-
 #include "idx.h"
 #include "symbol.h"
 #include "lst.h"
@@ -31,6 +27,11 @@
 #include "operators.h"
 #include "archive.h"
 #include "utils.h"
+#include "hash_seed.h"
+
+#include <iostream>
+#include <sstream>
+#include <stdexcept>
 
 namespace GiNaC {
 
@@ -54,23 +55,21 @@ GINAC_IMPLEMENT_REGISTERED_CLASS_OPT(spinidx, varidx,
 // default constructor
 //////////
 
-idx::idx() : inherited(&idx::tinfo_static) {}
+idx::idx() {}
 
 varidx::varidx() : covariant(false)
 {
-	tinfo_key = &varidx::tinfo_static;
 }
 
 spinidx::spinidx() : dotted(false)
 {
-	tinfo_key = &spinidx::tinfo_static;
 }
 
 //////////
 // other constructors
 //////////
 
-idx::idx(const ex & v, const ex & d) : inherited(&idx::tinfo_static), value(v), dim(d)
+idx::idx(const ex & v, const ex & d) :  value(v), dim(d)
 {
 	if (is_dim_numeric())
 		if (!dim.info(info_flags::posint))
@@ -79,33 +78,37 @@ idx::idx(const ex & v, const ex & d) : inherited(&idx::tinfo_static), value(v),
 
 varidx::varidx(const ex & v, const ex & d, bool cov) : inherited(v, d), covariant(cov)
 {
-	tinfo_key = &varidx::tinfo_static;
 }
 
 spinidx::spinidx(const ex & v, const ex & d, bool cov, bool dot) : inherited(v, d, cov), dotted(dot)
 {
-	tinfo_key = &spinidx::tinfo_static;
 }
 
 //////////
 // archiving
 //////////
 
-idx::idx(const archive_node &n, lst &sym_lst) : inherited(n, sym_lst)
+void idx::read_archive(const archive_node& n, lst& sym_lst) 
 {
+	inherited::read_archive(n, sym_lst);
 	n.find_ex("value", value, sym_lst);
 	n.find_ex("dim", dim, sym_lst);
 }
+GINAC_BIND_UNARCHIVER(idx);
 
-varidx::varidx(const archive_node &n, lst &sym_lst) : inherited(n, sym_lst)
+void varidx::read_archive(const archive_node& n, lst& sym_lst)
 {
+	inherited::read_archive(n, sym_lst);
 	n.find_bool("covariant", covariant);
 }
+GINAC_BIND_UNARCHIVER(varidx);
 
-spinidx::spinidx(const archive_node &n, lst &sym_lst) : inherited(n, sym_lst)
+void spinidx::read_archive(const archive_node& n, lst& sym_lst)
 {
+	inherited::read_archive(n, sym_lst);
 	n.find_bool("dotted", dotted);
 }
+GINAC_BIND_UNARCHIVER(spinidx);
 
 void idx::archive(archive_node &n) const
 {
@@ -126,10 +129,6 @@ void spinidx::archive(archive_node &n) const
 	n.add_bool("dotted", dotted);
 }
 
-DEFAULT_UNARCHIVE(idx)
-DEFAULT_UNARCHIVE(varidx)
-DEFAULT_UNARCHIVE(spinidx)
-
 //////////
 // functions overriding virtual functions from base classes
 //////////
@@ -352,7 +351,7 @@ unsigned idx::calchash() const
 	// hash keys. That is, the hash values must not depend on the index
 	// dimensions or other attributes (variance etc.).
 	// The compare_same_type() methods will take care of the rest.
-	unsigned v = golden_ratio_hash((p_int)tinfo());
+	unsigned v = make_hash_seed(typeid(*this));
 	v = rotate_left(v);
 	v ^= value.gethash();
 
@@ -504,7 +503,7 @@ ex spinidx::toggle_variance_dot() const
 bool is_dummy_pair(const idx & i1, const idx & i2)
 {
 	// The indices must be of exactly the same type
-	if (i1.tinfo() != i2.tinfo())
+	if (typeid(i1) != typeid(i2))
 		return false;
 
 	// Same type, let the indices decide whether they are paired
