@@ -6,7 +6,7 @@
  *  computation, square-free factorization and rational function normalization. */
 
 /*
- *  GiNaC Copyright (C) 1999-2005 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2010 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -23,13 +23,46 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#ifndef __GINAC_NORMAL_H__
-#define __GINAC_NORMAL_H__
+#ifndef GINAC_NORMAL_H
+#define GINAC_NORMAL_H
 
 #include "lst.h"
 
 namespace GiNaC {
 
+/**
+ * Flags to control the behaviour of gcd() and friends
+ */
+struct gcd_options
+{
+	enum {
+		/**
+		 * Usually GiNaC tries heuristic GCD first, because typically
+		 * it's much faster than anything else. Even if heuristic
+		 * algorithm fails, the overhead is negligible w.r.t. cost
+		 * of computing the GCD by some other method. However, some
+		 * people dislike it, so here's a flag which tells GiNaC
+		 * to NOT use the heuristic algorithm.
+		 */
+		no_heur_gcd = 2,
+		/**
+		 * GiNaC tries to avoid expanding expressions when computing
+		 * GCDs. This is a good idea, but some people dislike it.
+		 * Hence the flag to disable special handling of partially
+		 * factored polynomials. DON'T SET THIS unless you *really*
+		 * know what are you doing!
+		 */
+		no_part_factored = 4,
+		/**
+		 * By default GiNaC uses modular GCD algorithm. Typically
+		 * it's much faster than PRS (pseudo remainder sequence)
+		 * algorithm. This flag forces GiNaC to use PRS algorithm
+		 */
+		use_sr_gcd = 8
+
+	};
+};
+
 class ex;
 class symbol;
 
@@ -52,7 +85,8 @@ extern ex sprem(const ex &a, const ex &b, const ex &x, bool check_args = true);
 extern bool divide(const ex &a, const ex &b, ex &q, bool check_args = true);
 
 // Polynomial GCD in Z[X], cofactors are returned in ca and cb, if desired
-extern ex gcd(const ex &a, const ex &b, ex *ca = NULL, ex *cb = NULL, bool check_args = true);
+extern ex gcd(const ex &a, const ex &b, ex *ca = NULL, ex *cb = NULL,
+	      bool check_args = true, unsigned options = 0);
 
 // Polynomial LCM in Z[X]
 extern ex lcm(const ex &a, const ex &b, bool check_args = true);
@@ -71,4 +105,4 @@ extern ex resultant(const ex & e1, const ex & e2, const ex & s);
 
 } // namespace GiNaC
 
-#endif // ndef __GINAC_NORMAL_H__
+#endif // ndef GINAC_NORMAL_H
