@@ -3,7 +3,7 @@
  *  Interface to GiNaC's symbolic exponentiation (basis^exponent). */
 
 /*
- *  GiNaC Copyright (C) 1999-2007 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2010 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,11 +20,12 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#ifndef __GINAC_POWER_H__
-#define __GINAC_POWER_H__
+#ifndef GINAC_POWER_H
+#define GINAC_POWER_H
 
 #include "basic.h"
 #include "ex.h"
+#include "archive.h"
 
 namespace GiNaC {
 
@@ -44,8 +45,8 @@ class power : public basic
 	
 	// other constructors
 public:
-	power(const ex & lh, const ex & rh) : inherited(&power::tinfo_static), basis(lh), exponent(rh) {}
-	template<typename T> power(const ex & lh, const T & rh) : inherited(&power::tinfo_static), basis(lh), exponent(rh) {}
+	power(const ex & lh, const ex & rh) :  basis(lh), exponent(rh) {}
+	template<typename T> power(const ex & lh, const T & rh) :  basis(lh), exponent(rh) {}
 	
 	// functions overriding virtual functions from base classes
 public:
@@ -70,11 +71,15 @@ public:
 	ex conjugate() const;
 	ex real_part() const;
 	ex imag_part() const;
+	/** Save (a.k.a. serialize) object into archive. */
+	void archive(archive_node& n) const;
+	/** Read (a.k.a. deserialize) object from archive. */
+	void read_archive(const archive_node& n, lst& syms);
 protected:
 	ex derivative(const symbol & s) const;
 	ex eval_ncmul(const exvector & v) const;
 	unsigned return_type() const;
-	tinfo_t return_type_tinfo() const;
+	return_type_t return_type_tinfo() const;
 	ex expand(unsigned options = 0) const;
 	
 	// new virtual functions which can be overridden by derived classes
@@ -88,6 +93,7 @@ protected:
 	void do_print_csrc(const print_csrc & c, unsigned level) const;
 	void do_print_python(const print_python & c, unsigned level) const;
 	void do_print_python_repr(const print_python_repr & c, unsigned level) const;
+	void do_print_csrc_cl_N(const print_csrc_cl_N & c, unsigned level) const;
 
 	ex expand_add(const add & a, int n, unsigned options) const;
 	ex expand_add_2(const add & a, unsigned options) const;
@@ -99,6 +105,7 @@ protected:
 	ex basis;
 	ex exponent;
 };
+GINAC_DECLARE_UNARCHIVER(power); 
 
 // wrapper functions
 
@@ -125,4 +132,4 @@ inline ex sqrt(const ex & a)
 
 } // namespace GiNaC
 
-#endif // ndef __GINAC_POWER_H__
+#endif // ndef GINAC_POWER_H
