@@ -3,7 +3,7 @@
  *  GiNaC's class registrar (for class basic and all classes derived from it). */
 
 /*
- *  GiNaC Copyright (C) 1999-2007 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2010 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,16 +20,17 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#ifndef __GINAC_REGISTRAR_H__
-#define __GINAC_REGISTRAR_H__
-
-#include <string>
-#include <list>
-#include <vector>
+#ifndef GINAC_REGISTRAR_H
+#define GINAC_REGISTRAR_H
 
 #include "class_info.h"
 #include "print.h"
 
+#include <list>
+#include <string>
+#include <typeinfo>
+#include <vector>
+
 namespace GiNaC {
 
 class ex;
@@ -38,21 +39,53 @@ class archive_node;
 template <template <class T, class = std::allocator<T> > class> class container;
 typedef container<std::list> lst;
 
+/** To distinguish between different kinds of non-commutative objects */
+struct return_type_t
+{
+	/// to distinguish between non-commutative objects of different type.
+	std::type_info const* tinfo; 
+	/// to distinguish between non-commutative objects of the same type.
+	/// Think of gamma matrices with different represenation labels.
+	unsigned rl;
 
-/** Unarchiving function (static member function of every GiNaC class). */
-typedef ex (*unarch_func)(const archive_node &n, lst &sym_lst);
+	/// Strict weak ordering (so one can put return_type_t's into
+	/// a STL container).
+	inline bool operator<(const return_type_t& other) const
+	{
+		if (tinfo->before(*other.tinfo))
+			return true;
+		return rl < other.rl;
+	}
+	inline bool operator==(const return_type_t& other) const
+	{
+		if (*tinfo != *(other.tinfo))
+			return false;
+		return rl == other.rl;
+	}
+	inline bool operator!=(const return_type_t& other) const
+	{
+		return ! (operator==(other));
+	}
+};
 
+template<typename T> inline return_type_t make_return_type_t(const unsigned rl = 0)
+{
+	return_type_t ret;
+	ret.rl = rl;
+	ret.tinfo = &typeid(T);
+	return ret;
+}
 
 /** This class stores information about a registered GiNaC class. */
 class registered_class_options {
 public:
-	registered_class_options(const char *n, const char *p, unsigned ti, unarch_func f)
-	 : name(n), parent_name(p), tinfo_key(ti), unarchive(f) {}
+	registered_class_options(const char *n, const char *p, 
+		                 const std::type_info& ti)
+	 : name(n), parent_name(p), tinfo_key(&ti) { }
 
 	const char *get_name() const { return name; }
 	const char *get_parent_name() const { return parent_name; }
-	unsigned get_id() const { return tinfo_key; }
-	unarch_func get_unarch_func() const { return unarchive; }
+	std::type_info const* get_id() const { return tinfo_key; }
 	const std::vector<print_functor> &get_print_dispatch_table() const { return print_dispatch_table; }
 
 	template <class Ctx, class T, class C>
@@ -86,8 +119,7 @@ public:
 private:
 	const char *name;         /**< Class name. */
 	const char *parent_name;  /**< Name of superclass. */
-	unsigned tinfo_key;       /**< TINFO_* key. */
-	unarch_func unarchive;    /**< Pointer to unarchiving function. */
+	std::type_info const* tinfo_key;        /**< Type information key. */
 	std::vector<print_functor> print_dispatch_table; /**< Method table for print() dispatch */
 };
 
@@ -105,14 +137,10 @@ public: \
 	virtual const GiNaC::registered_class_info &get_class_info() const { return classname::get_class_info_static(); } \
 	virtual GiNaC::registered_class_info &get_class_info() { return classname::get_class_info_static(); } \
 	virtual const char *class_name() const { return classname::get_class_info_static().options.get_name(); } \
-	\
-	classname(const GiNaC::archive_node &n, GiNaC::lst &sym_lst); \
-	virtual void archive(GiNaC::archive_node &n) const; \
-	static GiNaC::ex unarchive(const GiNaC::archive_node &n, GiNaC::lst &sym_lst); \
-	\
 	class visitor { \
 	public: \
 		virtual void visit(const classname &) = 0; \
+		virtual ~visitor() {}; \
 	};
 
 /** Macro for inclusion in the declaration of each registered class.
@@ -136,21 +164,20 @@ protected: \
 	virtual int compare_same_type(const GiNaC::basic & other) const; \
 private:
 
+
 /** Macro for inclusion in the implementation of each registered class. */
 #define GINAC_IMPLEMENT_REGISTERED_CLASS(classname, supername) \
-	GiNaC::registered_class_info classname::reg_info = GiNaC::registered_class_info(GiNaC::registered_class_options(#classname, #supername, TINFO_##classname, &classname::unarchive));
+	GiNaC::registered_class_info classname::reg_info = GiNaC::registered_class_info(GiNaC::registered_class_options(#classname, #supername, typeid(classname))); 
 
 /** Macro for inclusion in the implementation of each registered class.
  *  Additional options can be specified. */
 #define GINAC_IMPLEMENT_REGISTERED_CLASS_OPT(classname, supername, options) \
-	GiNaC::registered_class_info classname::reg_info = GiNaC::registered_class_info(GiNaC::registered_class_options(#classname, #supername, TINFO_##classname, &classname::unarchive).options);
+	GiNaC::registered_class_info classname::reg_info = GiNaC::registered_class_info(GiNaC::registered_class_options(#classname, #supername, typeid(classname)).options);
 
-
-/** Find TINFO_* key by class name. */
-extern unsigned find_tinfo_key(const std::string &class_name);
-
-/** Find unarchiving function by class name. */
-extern unarch_func find_unarch_func(const std::string &class_name);
+/** Macro for inclusion in the implementation of each registered class.
+ *  Additional options can be specified. */
+#define GINAC_IMPLEMENT_REGISTERED_CLASS_OPT_T(classname, supername, options) \
+	GiNaC::registered_class_info classname::reg_info = GiNaC::registered_class_info(GiNaC::registered_class_options(#classname, #supername, typeid(classname)).options);
 
 
 /** Add or replace a print method. */
@@ -170,4 +197,4 @@ extern void set_print_func(void (T::*f)(const C &, unsigned))
 
 } // namespace GiNaC
 
-#endif // ndef __GINAC_REGISTRAR_H__
+#endif // ndef GINAC_REGISTRAR_H
