@@ -3,7 +3,7 @@
  *  Interface to relations between expressions. */
 
 /*
- *  GiNaC Copyright (C) 1999-2007 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2010 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,11 +20,12 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#ifndef __GINAC_RELATIONAL_H__
-#define __GINAC_RELATIONAL_H__
+#ifndef GINAC_RELATIONAL_H
+#define GINAC_RELATIONAL_H
 
 #include "basic.h"
 #include "ex.h"
+#include "archive.h"
 
 namespace GiNaC {
 
@@ -59,11 +60,15 @@ public:
 	ex subs(const exmap & m, unsigned options = 0) const;
 	ex eval(int level=0) const;
 
+	/** Save (a.k.a. serialize) object into archive. */
+	void archive(archive_node& n) const;
+	/** Read (a.k.a. deserialize) object from archive. */
+	void read_archive(const archive_node& n, lst& syms);
 protected:
 	ex eval_ncmul(const exvector & v) const;
 	bool match_same_type(const basic & other) const;
 	unsigned return_type() const;
-	tinfo_t return_type_tinfo() const;
+	return_type_t return_type_tinfo() const;
 	unsigned calchash() const;
 
 	// new virtual functions which can be overridden by derived classes
@@ -100,6 +105,7 @@ protected:
 	ex rh;
 	operators o;
 };
+GINAC_DECLARE_UNARCHIVER(relational); 
 
 // utility functions
 
@@ -111,4 +117,4 @@ inline relational::safe_bool relational::operator!() const
 
 } // namespace GiNaC
 
-#endif // ndef __GINAC_RELATIONAL_H__
+#endif // ndef GINAC_RELATIONAL_H
