@@ -4,7 +4,7 @@
  *  in GiNaC functions */
 
 /*
- *  GiNaC Copyright (C) 1999-2005 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2010 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,12 +21,12 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#ifndef __GINAC_REMEMBER_H__
-#define __GINAC_REMEMBER_H__
+#ifndef GINAC_REMEMBER_H
+#define GINAC_REMEMBER_H
 
 #include <iosfwd>
-#include <vector>
 #include <list>
+#include <vector>
 
 namespace GiNaC {
 
@@ -98,4 +98,4 @@ protected:
 
 } // namespace GiNaC
 
-#endif // ndef __GINAC_REMEMBER_H__
+#endif // ndef GINAC_REMEMBER_H
