@@ -1,10 +1,9 @@
-
-/* A Bison parser, made by GNU Bison 2.4.1.  */
+/* A Bison parser, made by GNU Bison 2.4.2.  */
 
 /* Skeleton implementation for Bison's Yacc-like parsers in C
    
-      Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006
-   Free Software Foundation, Inc.
+      Copyright (C) 1984, 1989-1990, 2000-2006, 2009-2010 Free Software
+   Foundation, Inc.
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -46,7 +45,7 @@
 #define YYBISON 1
 
 /* Bison version.  */
-#define YYBISON_VERSION "2.4.1"
+#define YYBISON_VERSION "2.4.2"
 
 /* Skeleton name.  */
 #define YYSKELETON_NAME "yacc.c"
@@ -156,7 +155,7 @@ static void print_help_topics(void);
 
 
 /* Line 189 of yacc.c  */
-#line 160 "ginsh_parser.cc"
+#line 159 "ginsh_parser.cc"
 
 /* Enabling traces.  */
 #ifndef YYDEBUG
@@ -252,7 +251,7 @@ typedef int YYSTYPE;
 
 
 /* Line 264 of yacc.c  */
-#line 256 "ginsh_parser.cc"
+#line 255 "ginsh_parser.cc"
 
 #ifdef short
 # undef short
@@ -302,7 +301,7 @@ typedef short int yytype_int16;
 #define YYSIZE_MAXIMUM ((YYSIZE_T) -1)
 
 #ifndef YY_
-# if YYENABLE_NLS
+# if defined YYENABLE_NLS && YYENABLE_NLS
 #  if ENABLE_NLS
 #   include <libintl.h> /* INFRINGES ON USER NAME SPACE */
 #   define YY_(msgid) dgettext ("bison-runtime", msgid)
@@ -785,9 +784,18 @@ static const yytype_uint8 yystos[] =
 
 /* Like YYERROR except do call yyerror.  This remains here temporarily
    to ease the transition to the new meaning of YYERROR, for GCC.
-   Once GCC version 2 has supplanted version 1, this can go.  */
+   Once GCC version 2 has supplanted version 1, this can go.  However,
+   YYFAIL appears to be in use.  Nevertheless, it is formally deprecated
+   in Bison 2.4.2's NEWS entry, where a plan to phase it out is
+   discussed.  */
 
 #define YYFAIL		goto yyerrlab
+#if defined YYFAIL
+  /* This is here to suppress warnings from the GCC cpp's
+     -Wunused-macros.  Normally we don't worry about that warning, but
+     some users do, and we want to make it easy for users to remove
+     YYFAIL uses, which will produce warnings from Bison 2.5.  */
+#endif
 
 #define YYRECOVERING()  (!!yyerrstatus)
 
@@ -844,7 +852,7 @@ while (YYID (0))
    we won't break user code: when these are the locations we know.  */
 
 #ifndef YY_LOCATION_PRINT
-# if YYLTYPE_IS_TRIVIAL
+# if defined YYLTYPE_IS_TRIVIAL && YYLTYPE_IS_TRIVIAL
 #  define YY_LOCATION_PRINT(File, Loc)			\
      fprintf (File, "%d.%d-%d.%d",			\
 	      (Loc).first_line, (Loc).first_column,	\
@@ -1583,7 +1591,7 @@ yyreduce:
     {
         case 5:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 146 "ginsh_parser.yy"
     {
 		try {
@@ -1598,7 +1606,7 @@ yyreduce:
 
   case 6:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 155 "ginsh_parser.yy"
     {
 		try {
@@ -1612,7 +1620,7 @@ yyreduce:
 
   case 7:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 163 "ginsh_parser.yy"
     {
 		try {
@@ -1626,7 +1634,7 @@ yyreduce:
 
   case 8:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 171 "ginsh_parser.yy"
     {
 		try {
@@ -1646,7 +1654,7 @@ yyreduce:
 
   case 9:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 185 "ginsh_parser.yy"
     {
 		try {
@@ -1660,7 +1668,7 @@ yyreduce:
 
   case 10:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 193 "ginsh_parser.yy"
     {
 		try {
@@ -1674,63 +1682,63 @@ yyreduce:
 
   case 11:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 201 "ginsh_parser.yy"
     {print_help(ex_to<symbol>((yyvsp[(2) - (2)])).get_name());}
     break;
 
   case 12:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 202 "ginsh_parser.yy"
     {print_help("time");}
     break;
 
   case 13:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 203 "ginsh_parser.yy"
     {print_help("print");}
     break;
 
   case 14:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 204 "ginsh_parser.yy"
     {print_help("iprint");}
     break;
 
   case 15:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 205 "ginsh_parser.yy"
     {print_help("print_latex");}
     break;
 
   case 16:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 206 "ginsh_parser.yy"
     {print_help("print_csrc");}
     break;
 
   case 17:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 207 "ginsh_parser.yy"
     {print_help_topics();}
     break;
 
   case 18:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 208 "ginsh_parser.yy"
     {YYACCEPT;}
     break;
 
   case 19:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 209 "ginsh_parser.yy"
     {
 		cout << "This program is free software; you can redistribute it and/or modify it under\n";
@@ -1749,28 +1757,28 @@ yyreduce:
 
   case 20:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 222 "ginsh_parser.yy"
     {cout << "Nothing happens.\n";}
     break;
 
   case 21:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 223 "ginsh_parser.yy"
     {cout << "You're not carrying anything.\n";}
     break;
 
   case 22:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 224 "ginsh_parser.yy"
     {cout << "You're in a twisty little maze of passages, all alike.\n";}
     break;
 
   case 23:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 225 "ginsh_parser.yy"
     {
 		cout << "If you were to quit now, you would score ";
@@ -1781,56 +1789,56 @@ yyreduce:
 
   case 24:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 230 "ginsh_parser.yy"
     { symboltype = domain::real; }
     break;
 
   case 25:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 231 "ginsh_parser.yy"
     { symboltype = domain::complex; }
     break;
 
   case 26:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 232 "ginsh_parser.yy"
     { START_TIMER }
     break;
 
   case 27:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 232 "ginsh_parser.yy"
     { STOP_TIMER PRINT_TIME_USED }
     break;
 
   case 28:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 233 "ginsh_parser.yy"
     {yyclearin; yyerrok;}
     break;
 
   case 29:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 234 "ginsh_parser.yy"
     {yyclearin; yyerrok;}
     break;
 
   case 30:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 237 "ginsh_parser.yy"
     {(yyval) = (yyvsp[(1) - (1)]);}
     break;
 
   case 31:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 238 "ginsh_parser.yy"
     {
 		exmap::const_iterator i = assigned_symbol_table.find((yyvsp[(1) - (1)]));
@@ -1843,49 +1851,49 @@ yyreduce:
 
   case 32:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 245 "ginsh_parser.yy"
     {(yyval) = (yyvsp[(2) - (3)]);}
     break;
 
   case 33:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 246 "ginsh_parser.yy"
     {(yyval) = (yyvsp[(1) - (1)]);}
     break;
 
   case 34:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 247 "ginsh_parser.yy"
     {(yyval) = (yyvsp[(1) - (1)]);}
     break;
 
   case 35:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 248 "ginsh_parser.yy"
     {(yyval) = exstack[0];}
     break;
 
   case 36:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 249 "ginsh_parser.yy"
     {(yyval) = exstack[1];}
     break;
 
   case 37:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 250 "ginsh_parser.yy"
     {(yyval) = exstack[2];}
     break;
 
   case 38:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 251 "ginsh_parser.yy"
     {
 		fcn_tab::const_iterator i = find_function((yyvsp[(1) - (4)]), (yyvsp[(3) - (4)]).nops());
@@ -1899,211 +1907,211 @@ yyreduce:
 
   case 39:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 259 "ginsh_parser.yy"
     {(yyval) = (yyvsp[(3) - (3)]); Digits = ex_to<numeric>((yyvsp[(3) - (3)])).to_int();}
     break;
 
   case 40:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 260 "ginsh_parser.yy"
     {(yyval) = (yyvsp[(3) - (3)]); assigned_symbol_table[(yyvsp[(1) - (3)])] = (yyvsp[(3) - (3)]); }
     break;
 
   case 41:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 261 "ginsh_parser.yy"
     {(yyval) = (yyvsp[(1) - (3)]) == (yyvsp[(3) - (3)]);}
     break;
 
   case 42:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 262 "ginsh_parser.yy"
     {(yyval) = (yyvsp[(1) - (3)]) != (yyvsp[(3) - (3)]);}
     break;
 
   case 43:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 263 "ginsh_parser.yy"
     {(yyval) = (yyvsp[(1) - (3)]) < (yyvsp[(3) - (3)]);}
     break;
 
   case 44:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 264 "ginsh_parser.yy"
     {(yyval) = (yyvsp[(1) - (3)]) <= (yyvsp[(3) - (3)]);}
     break;
 
   case 45:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 265 "ginsh_parser.yy"
     {(yyval) = (yyvsp[(1) - (3)]) > (yyvsp[(3) - (3)]);}
     break;
 
   case 46:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 266 "ginsh_parser.yy"
     {(yyval) = (yyvsp[(1) - (3)]) >= (yyvsp[(3) - (3)]);}
     break;
 
   case 47:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 267 "ginsh_parser.yy"
     {(yyval) = (yyvsp[(1) - (3)]) + (yyvsp[(3) - (3)]);}
     break;
 
   case 48:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 268 "ginsh_parser.yy"
     {(yyval) = (yyvsp[(1) - (3)]) - (yyvsp[(3) - (3)]);}
     break;
 
   case 49:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 269 "ginsh_parser.yy"
     {(yyval) = (yyvsp[(1) - (3)]) * (yyvsp[(3) - (3)]);}
     break;
 
   case 50:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 270 "ginsh_parser.yy"
     {(yyval) = (yyvsp[(1) - (3)]) / (yyvsp[(3) - (3)]);}
     break;
 
   case 51:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 271 "ginsh_parser.yy"
     {(yyval) = -(yyvsp[(2) - (2)]);}
     break;
 
   case 52:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 272 "ginsh_parser.yy"
     {(yyval) = (yyvsp[(2) - (2)]);}
     break;
 
   case 53:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 273 "ginsh_parser.yy"
     {(yyval) = power((yyvsp[(1) - (3)]), (yyvsp[(3) - (3)]));}
     break;
 
   case 54:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 274 "ginsh_parser.yy"
     {(yyval) = factorial((yyvsp[(1) - (2)]));}
     break;
 
   case 55:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 275 "ginsh_parser.yy"
     {(yyval) = (yyvsp[(2) - (3)]);}
     break;
 
   case 56:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 276 "ginsh_parser.yy"
     {(yyval) = (yyvsp[(2) - (3)]);}
     break;
 
   case 57:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 277 "ginsh_parser.yy"
     {(yyval) = lst_to_matrix(ex_to<lst>((yyvsp[(2) - (3)])));}
     break;
 
   case 58:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 280 "ginsh_parser.yy"
     {(yyval) = exprseq((yyvsp[(1) - (1)]));}
     break;
 
   case 59:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 281 "ginsh_parser.yy"
     {exprseq es(ex_to<exprseq>((yyvsp[(1) - (3)]))); (yyval) = es.append((yyvsp[(3) - (3)]));}
     break;
 
   case 60:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 284 "ginsh_parser.yy"
     {(yyval) = *new lst;}
     break;
 
   case 61:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 285 "ginsh_parser.yy"
     {(yyval) = (yyvsp[(1) - (1)]);}
     break;
 
   case 62:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 288 "ginsh_parser.yy"
     {(yyval) = lst((yyvsp[(1) - (1)]));}
     break;
 
   case 63:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 289 "ginsh_parser.yy"
     {lst l(ex_to<lst>((yyvsp[(1) - (3)]))); (yyval) = l.append((yyvsp[(3) - (3)]));}
     break;
 
   case 64:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 292 "ginsh_parser.yy"
     {(yyval) = lst((yyvsp[(2) - (3)]));}
     break;
 
   case 65:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 293 "ginsh_parser.yy"
     {lst l(ex_to<lst>((yyvsp[(1) - (5)]))); (yyval) = l.append((yyvsp[(4) - (5)]));}
     break;
 
   case 66:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 296 "ginsh_parser.yy"
     {(yyval) = lst((yyvsp[(1) - (1)]));}
     break;
 
   case 67:
 
-/* Line 1455 of yacc.c  */
+/* Line 1464 of yacc.c  */
 #line 297 "ginsh_parser.yy"
     {lst l(ex_to<lst>((yyvsp[(1) - (3)]))); (yyval) = l.append((yyvsp[(3) - (3)]));}
     break;
 
 
 
-/* Line 1455 of yacc.c  */
-#line 2107 "ginsh_parser.cc"
+/* Line 1464 of yacc.c  */
+#line 2115 "ginsh_parser.cc"
       default: break;
     }
   YY_SYMBOL_PRINT ("-> $$ =", yyr1[yyn], &yyval, &yyloc);
@@ -2314,7 +2322,7 @@ yyreturn:
 
 
 
-/* Line 1675 of yacc.c  */
+/* Line 1684 of yacc.c  */
 #line 305 "ginsh_parser.yy"
 
 // Error print routine
@@ -2741,7 +2749,7 @@ static void insert_fcns(const fcn_init *p)
 
 static ex f_ginac_function(const exprseq &es, int serial)
 {
-	return function(serial, es).eval(1);
+	return GiNaC::function(serial, es).eval(1);
 }
 
 // All registered GiNaC functions
@@ -2913,7 +2921,7 @@ static void ginsh_readline_init(char* name)
 void greeting(void)
 {
     cout << "ginsh - GiNaC Interactive Shell (" << PACKAGE << " V" << VERSION << ")" << endl;
-    cout << "  __,  _______  Copyright (C) 1999-2009 Johannes Gutenberg University Mainz,\n"
+    cout << "  __,  _______  Copyright (C) 1999-2010 Johannes Gutenberg University Mainz,\n"
          << " (__) *       | Germany.  This is free software with ABSOLUTELY NO WARRANTY.\n"
          << "  ._) i N a C | You are welcome to redistribute it under certain conditions.\n"
          << "<-------------' For details type `warranty;'.\n" << endl;
